@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,8 +22,8 @@
 #include "LogBackend.hxx"
 #include "Log.hxx"
 #include "config/Param.hxx"
-#include "config/ConfigGlobal.hxx"
-#include "config/ConfigOption.hxx"
+#include "config/Data.hxx"
+#include "config/Option.hxx"
 #include "fs/AllocatedPath.hxx"
 #include "fs/FileSystem.hxx"
 #include "util/Domain.hxx"
@@ -41,12 +41,13 @@
 #define LOG_DATE_BUF_SIZE 16
 #define LOG_DATE_LEN (LOG_DATE_BUF_SIZE - 1)
 
+gcc_unused
 static constexpr Domain log_domain("log");
 
 #ifndef ANDROID
 
 static int out_fd = -1;
-static AllocatedPath out_path = AllocatedPath::Null();
+static AllocatedPath out_path = nullptr;
 
 static void redirect_logs(int fd)
 {
@@ -62,7 +63,7 @@ open_log_file(void)
 {
 	assert(!out_path.IsNull());
 
-	return OpenFile(out_path, O_CREAT | O_WRONLY | O_APPEND, 0666);
+	return OpenFile(out_path, O_CREAT | O_WRONLY | O_APPEND, 0666).Steal();
 }
 
 static void
@@ -72,7 +73,7 @@ log_init_file(int line)
 
 	out_fd = open_log_file();
 	if (out_fd < 0) {
-#ifdef WIN32
+#ifdef _WIN32
 		const std::string out_path_utf8 = out_path.ToUTF8();
 		throw FormatRuntimeError("failed to open log file \"%s\" (config line %d)",
 					 out_path_utf8.c_str(), line);
@@ -118,22 +119,23 @@ log_early_init(bool verbose)
 }
 
 void
-log_init(bool verbose, bool use_stdout)
+log_init(const ConfigData &config, bool verbose, bool use_stdout)
 {
 #ifdef ANDROID
+	(void)config;
 	(void)verbose;
 	(void)use_stdout;
 #else
 	if (verbose)
 		SetLogThreshold(LogLevel::DEBUG);
-	else if (const auto &param = config_get_param(ConfigOption::LOG_LEVEL))
+	else if (const auto &param = config.GetParam(ConfigOption::LOG_LEVEL))
 		SetLogThreshold(parse_log_level(param->value.c_str(),
 						param->line));
 
 	if (use_stdout) {
 		out_fd = STDOUT_FILENO;
 	} else {
-		const auto *param = config_get_param(ConfigOption::LOG_FILE);
+		const auto *param = config.GetParam(ConfigOption::LOG_FILE);
 		if (param == nullptr) {
 			/* no configuration: default to syslog (if
 			   available) */
@@ -169,7 +171,7 @@ log_deinit(void)
 {
 #ifndef ANDROID
 	close_log_files();
-	out_path = AllocatedPath::Null();
+	out_path = nullptr;
 #endif
 }
 
@@ -182,7 +184,7 @@ void setup_log_output()
 	fflush(nullptr);
 
 	if (out_fd < 0) {
-#ifdef WIN32
+#ifdef _WIN32
 		return;
 #else
 		out_fd = open("/dev/null", O_WRONLY);
