@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,22 +24,21 @@
 #include "PlaylistFile.hxx"
 #include "MusicChunk.hxx"
 #include "StateFile.hxx"
-#include "player/Thread.hxx"
 #include "Mapper.hxx"
 #include "Permission.hxx"
 #include "Listen.hxx"
+#include "client/Listener.hxx"
 #include "client/Client.hxx"
 #include "client/ClientList.hxx"
 #include "command/AllCommands.hxx"
 #include "Partition.hxx"
-#include "tag/TagConfig.hxx"
+#include "tag/Config.hxx"
 #include "ReplayGainGlobal.hxx"
 #include "Idle.hxx"
 #include "Log.hxx"
 #include "LogInit.hxx"
 #include "input/Init.hxx"
 #include "event/Loop.hxx"
-#include "IOThread.hxx"
 #include "fs/AllocatedPath.hxx"
 #include "fs/Config.hxx"
 #include "playlist/PlaylistRegistry.hxx"
@@ -48,16 +47,19 @@
 #include "AudioParser.hxx"
 #include "pcm/PcmConvert.hxx"
 #include "unix/SignalHandlers.hxx"
-#include "system/FatalError.hxx"
 #include "thread/Slack.hxx"
 #include "net/Init.hxx"
 #include "lib/icu/Init.hxx"
-#include "config/ConfigGlobal.hxx"
+#include "config/File.hxx"
+#include "config/Check.hxx"
+#include "config/Data.hxx"
 #include "config/Param.hxx"
-#include "config/ConfigDefaults.hxx"
-#include "config/ConfigOption.hxx"
-#include "config/ConfigError.hxx"
+#include "config/Path.hxx"
+#include "config/Defaults.hxx"
+#include "config/Option.hxx"
+#include "config/Domain.hxx"
 #include "util/RuntimeError.hxx"
+#include "util/ScopeExit.hxx"
 
 #ifdef ENABLE_DAEMON
 #include "unix/Daemon.hxx"
@@ -93,11 +95,14 @@
 #include "android/Environment.hxx"
 #include "android/Context.hxx"
 #include "android/LogListener.hxx"
-#include "fs/StandardDirectory.hxx"
 #include "fs/FileSystem.hxx"
 #include "org_musicpd_Bridge.h"
 #endif
 
+#ifdef ENABLE_DBUS
+#include "lib/dbus/Init.hxx"
+#endif
+
 #ifdef ENABLE_SYSTEMD_DAEMON
 #include <systemd/sd-daemon.h>
 #endif
@@ -115,18 +120,10 @@ static constexpr size_t MEGABYTE = 1024 * KILOBYTE;
 
 static constexpr size_t DEFAULT_BUFFER_SIZE = 4 * MEGABYTE;
 
-static
-#if GCC_OLDER_THAN(5,0)
-/* gcc 4.x has no "constexpr" for std::max() */
-const
-#else
-constexpr
-#endif
+static constexpr
 size_t MIN_BUFFER_SIZE = std::max(CHUNK_SIZE * 32,
 				  64 * KILOBYTE);
 
-static constexpr unsigned DEFAULT_BUFFER_BEFORE_PLAY = 10;
-
 #ifdef ANDROID
 Context *context;
 LogListener *logListener;
@@ -139,19 +136,20 @@ struct Config {
 };
 
 static Config
-LoadConfig()
+LoadConfig(const ConfigData &config)
 {
-	return {LoadReplayGainConfig()};
+	return {LoadReplayGainConfig(config)};
 }
 
 #ifdef ENABLE_DAEMON
 
 static void
-glue_daemonize_init(const struct options *options)
+glue_daemonize_init(const struct options *options,
+		    const ConfigData &config)
 {
-	daemonize_init(config_get_string(ConfigOption::USER, nullptr),
-		       config_get_string(ConfigOption::GROUP, nullptr),
-		       config_get_path(ConfigOption::PID_FILE));
+	daemonize_init(config.GetString(ConfigOption::USER),
+		       config.GetString(ConfigOption::GROUP),
+		       config.GetPath(ConfigOption::PID_FILE));
 
 	if (options->kill)
 		daemonize_kill();
@@ -160,23 +158,23 @@ glue_daemonize_init(const struct options *options)
 #endif
 
 static void
-glue_mapper_init()
+glue_mapper_init(const ConfigData &config)
 {
-	mapper_init(config_get_path(ConfigOption::PLAYLIST_DIR));
+	mapper_init(config.GetPath(ConfigOption::PLAYLIST_DIR));
 }
 
 #ifdef ENABLE_DATABASE
 
 static void
-InitStorage()
+InitStorage(const ConfigData &config, EventLoop &event_loop)
 {
-	Storage *storage = CreateConfiguredStorage(io_thread_get());
+	auto storage = CreateConfiguredStorage(config, event_loop);
 	if (storage == nullptr)
 		return;
 
 	CompositeStorage *composite = new CompositeStorage();
 	instance->storage = composite;
-	composite->Mount("", storage);
+	composite->Mount("", std::move(storage));
 }
 
 /**
@@ -185,15 +183,17 @@ InitStorage()
  * process has been daemonized.
  */
 static bool
-glue_db_init_and_load(void)
+glue_db_init_and_load(const ConfigData &config)
 {
 	instance->database =
-		CreateConfiguredDatabase(instance->event_loop, *instance);
+		CreateConfiguredDatabase(config, instance->event_loop,
+					 instance->io_thread.GetEventLoop(),
+					 *instance);
 	if (instance->database == nullptr)
 		return true;
 
 	if (instance->database->GetPlugin().flags & DatabasePlugin::FLAG_REQUIRE_STORAGE) {
-		InitStorage();
+		InitStorage(config, instance->io_thread.GetEventLoop());
 
 		if (instance->storage == nullptr) {
 			delete instance->database;
@@ -204,7 +204,7 @@ glue_db_init_and_load(void)
 			return true;
 		}
 	} else {
-		if (IsStorageConfigured())
+		if (IsStorageConfigured(config))
 			LogDefault(config_domain,
 				   "Ignoring the storage configuration "
 				   "because the database does not need it");
@@ -220,7 +220,8 @@ glue_db_init_and_load(void)
 		return true;
 
 	SimpleDatabase &db = *(SimpleDatabase *)instance->database;
-	instance->update = new UpdateService(instance->event_loop, db,
+	instance->update = new UpdateService(config,
+					     instance->event_loop, db,
 					     static_cast<CompositeStorage &>(*instance->storage),
 					     *instance);
 
@@ -229,9 +230,9 @@ glue_db_init_and_load(void)
 }
 
 static bool
-InitDatabaseAndStorage()
+InitDatabaseAndStorage(const ConfigData &config)
 {
-	const bool create_db = !glue_db_init_and_load();
+	const bool create_db = !glue_db_init_and_load(config);
 	return create_db;
 }
 
@@ -241,39 +242,28 @@ InitDatabaseAndStorage()
  * Configure and initialize the sticker subsystem.
  */
 static void
-glue_sticker_init()
+glue_sticker_init(const ConfigData &config)
 {
 #ifdef ENABLE_SQLITE
-	auto sticker_file = config_get_path(ConfigOption::STICKER_FILE);
+	auto sticker_file = config.GetPath(ConfigOption::STICKER_FILE);
 	if (sticker_file.IsNull())
 		return;
 
 	sticker_global_init(std::move(sticker_file));
+#else
+	(void)config;
 #endif
 }
 
 static void
-glue_state_file_init()
+glue_state_file_init(const ConfigData &raw_config)
 {
-	auto path_fs = config_get_path(ConfigOption::STATE_FILE);
-	if (path_fs.IsNull()) {
-#ifdef ANDROID
-		const auto cache_dir = GetUserCacheDir();
-		if (cache_dir.IsNull())
-			return;
-
-		path_fs = AllocatedPath::Build(cache_dir, "state");
-#else
+	StateFileConfig config(raw_config);
+	if (!config.IsEnabled())
 		return;
-#endif
-	}
 
-	const auto interval =
-		config_get_unsigned(ConfigOption::STATE_FILE_INTERVAL,
-				    StateFile::DEFAULT_INTERVAL);
-
-	instance->state_file = new StateFile(std::move(path_fs), interval,
-					     *instance->partition,
+	instance->state_file = new StateFile(std::move(config),
+					     instance->partitions.front(),
 					     instance->event_loop);
 	instance->state_file->Read();
 }
@@ -282,19 +272,20 @@ glue_state_file_init()
  * Initialize the decoder and player core, including the music pipe.
  */
 static void
-initialize_decoder_and_player(const ReplayGainConfig &replay_gain_config)
+initialize_decoder_and_player(const ConfigData &config,
+			      const ReplayGainConfig &replay_gain_config)
 {
 	const ConfigParam *param;
 
 	size_t buffer_size;
-	param = config_get_param(ConfigOption::AUDIO_BUFFER_SIZE);
+	param = config.GetParam(ConfigOption::AUDIO_BUFFER_SIZE);
 	if (param != nullptr) {
 		char *test;
 		long tmp = strtol(param->value.c_str(), &test, 10);
 		if (*test != '\0' || tmp <= 0 || tmp == LONG_MAX)
-			FormatFatalError("buffer size \"%s\" is not a "
-					 "positive integer, line %i",
-					 param->value.c_str(), param->line);
+			throw FormatRuntimeError("buffer size \"%s\" is not a "
+						 "positive integer, line %i",
+						 param->value.c_str(), param->line);
 		buffer_size = tmp * KILOBYTE;
 
 		if (buffer_size < MIN_BUFFER_SIZE) {
@@ -309,67 +300,37 @@ initialize_decoder_and_player(const ReplayGainConfig &replay_gain_config)
 	const unsigned buffered_chunks = buffer_size / CHUNK_SIZE;
 
 	if (buffered_chunks >= 1 << 15)
-		FormatFatalError("buffer size \"%lu\" is too big",
-				 (unsigned long)buffer_size);
-
-	float perc;
-	param = config_get_param(ConfigOption::BUFFER_BEFORE_PLAY);
-	if (param != nullptr) {
-		char *test;
-		perc = strtod(param->value.c_str(), &test);
-		if (*test != '%' || perc < 0 || perc > 100) {
-			FormatFatalError("buffered before play \"%s\" is not "
-					 "a positive percentage and less "
-					 "than 100 percent, line %i",
-					 param->value.c_str(), param->line);
-		}
-
-		if (perc > 80) {
-			/* this upper limit should avoid deadlocks
-			   which can occur because the DecoderThread
-			   cannot ever fill the music buffer to
-			   exactly 100%; a few chunks always need to
-			   be available to generate silence in
-			   Player::SendSilence() */
-			FormatError(config_domain,
-				    "buffer_before_play is too large (%f%%), capping at 80%%; please fix your configuration",
-				    perc);
-			perc = 80;
-		}
-	} else
-		perc = DEFAULT_BUFFER_BEFORE_PLAY;
-
-	unsigned buffered_before_play = (perc / 100) * buffered_chunks;
-	if (buffered_before_play > buffered_chunks)
-		buffered_before_play = buffered_chunks;
+		throw FormatRuntimeError("buffer size \"%lu\" is too big",
+					 (unsigned long)buffer_size);
 
 	const unsigned max_length =
-		config_get_positive(ConfigOption::MAX_PLAYLIST_LENGTH,
-				    DEFAULT_PLAYLIST_MAX_LENGTH);
+		config.GetPositive(ConfigOption::MAX_PLAYLIST_LENGTH,
+				   DEFAULT_PLAYLIST_MAX_LENGTH);
 
 	AudioFormat configured_audio_format = AudioFormat::Undefined();
-	param = config_get_param(ConfigOption::AUDIO_OUTPUT_FORMAT);
+	param = config.GetParam(ConfigOption::AUDIO_OUTPUT_FORMAT);
 	if (param != nullptr) {
 		try {
 			configured_audio_format = ParseAudioFormat(param->value.c_str(),
 								   true);
-		} catch (const std::runtime_error &) {
+		} catch (...) {
 			std::throw_with_nested(FormatRuntimeError("error parsing line %i",
 								  param->line));
 		}
 	}
 
-	instance->partition = new Partition(*instance,
-					    max_length,
-					    buffered_chunks,
-					    buffered_before_play,
-					    configured_audio_format,
-					    replay_gain_config);
+	instance->partitions.emplace_back(*instance,
+					  "default",
+					  max_length,
+					  buffered_chunks,
+					  configured_audio_format,
+					  replay_gain_config);
+	auto &partition = instance->partitions.back();
 
 	try {
-		param = config_get_param(ConfigOption::REPLAYGAIN);
+		param = config.GetParam(ConfigOption::REPLAYGAIN);
 		if (param != nullptr)
-			instance->partition->replay_gain_mode =
+			partition.replay_gain_mode =
 				FromString(param->value.c_str());
 	} catch (...) {
 		std::throw_with_nested(FormatRuntimeError("Failed to parse line %i",
@@ -377,6 +338,55 @@ initialize_decoder_and_player(const ReplayGainConfig &replay_gain_config)
 	}
 }
 
+inline void
+Instance::BeginShutdownUpdate() noexcept
+{
+#ifdef ENABLE_DATABASE
+#ifdef ENABLE_INOTIFY
+	mpd_inotify_finish();
+#endif
+
+	if (update != nullptr)
+		update->CancelAllAsync();
+#endif
+}
+
+inline void
+Instance::FinishShutdownUpdate() noexcept
+{
+#ifdef ENABLE_DATABASE
+	delete update;
+#endif
+}
+
+inline void
+Instance::ShutdownDatabase() noexcept
+{
+#ifdef ENABLE_DATABASE
+	if (instance->database != nullptr) {
+		instance->database->Close();
+		delete instance->database;
+	}
+
+	delete instance->storage;
+#endif
+}
+
+inline void
+Instance::BeginShutdownPartitions() noexcept
+{
+	for (auto &partition : partitions) {
+		partition.pc.Kill();
+		partition.listener.reset();
+	}
+}
+
+inline void
+Instance::FinishShutdownPartitions() noexcept
+{
+	partitions.clear();
+}
+
 void
 Instance::OnIdle(unsigned flags)
 {
@@ -391,7 +401,8 @@ Instance::OnIdle(unsigned flags)
 
 #ifndef ANDROID
 
-int main(int argc, char *argv[])
+int
+main(int argc, char *argv[]) noexcept
 {
 #ifdef _WIN32
 	return win32_main(argc, argv);
@@ -403,13 +414,12 @@ int main(int argc, char *argv[])
 #endif
 
 static int
-mpd_main_after_fork(const Config &config);
+mpd_main_after_fork(const ConfigData &raw_config,
+		    const Config &config);
 
-#ifdef ANDROID
-static inline
-#endif
-int mpd_main(int argc, char *argv[])
-try {
+static inline int
+MainOrThrow(int argc, char *argv[])
+{
 	struct options options;
 
 #ifdef ENABLE_DAEMON
@@ -424,12 +434,14 @@ try {
 #endif
 #endif
 
-	IcuInit();
-
+	const ScopeIcuInit icu_init;
 	const ScopeNetInit net_init;
 
-	io_thread_init();
-	config_global_init();
+#ifdef ENABLE_DBUS
+	const ODBus::ScopeInit dbus_init;
+#endif
+
+	ConfigData raw_config;
 
 #ifdef ANDROID
 	(void)argc;
@@ -438,29 +450,36 @@ try {
 	const auto sdcard = Environment::getExternalStorageDirectory();
 	if (!sdcard.IsNull()) {
 		const auto config_path =
-			AllocatedPath::Build(sdcard, "mpd.conf");
+			sdcard / Path::FromFS("mpd.conf");
 		if (FileExists(config_path))
-			ReadConfigFile(config_path);
+			ReadConfigFile(raw_config, config_path);
 	}
 #else
-	ParseCommandLine(argc, argv, &options);
+	ParseCommandLine(argc, argv, options, raw_config);
 #endif
 
-	const auto config = LoadConfig();
+	InitPathParser(raw_config);
+	const auto config = LoadConfig(raw_config);
 
 #ifdef ENABLE_DAEMON
-	glue_daemonize_init(&options);
+	glue_daemonize_init(&options, raw_config);
 #endif
 
-	TagLoadConfig();
+	TagLoadConfig(raw_config);
 
-	log_init(options.verbose, options.log_stderr);
+	log_init(raw_config, options.verbose, options.log_stderr);
 
 	instance = new Instance();
+	AtScopeExit() {
+		delete instance;
+		instance = nullptr;
+	};
 
 #ifdef ENABLE_NEIGHBOR_PLUGINS
 	instance->neighbors = new NeighborGlue();
-	instance->neighbors->Init(io_thread_get(), *instance);
+	instance->neighbors->Init(raw_config,
+				  instance->io_thread.GetEventLoop(),
+				  *instance);
 
 	if (instance->neighbors->IsEmpty()) {
 		delete instance->neighbors;
@@ -469,58 +488,74 @@ try {
 #endif
 
 	const unsigned max_clients =
-		config_get_positive(ConfigOption::MAX_CONN, 10);
+		raw_config.GetPositive(ConfigOption::MAX_CONN, 10);
 	instance->client_list = new ClientList(max_clients);
 
-	initialize_decoder_and_player(config.replay_gain);
+	initialize_decoder_and_player(raw_config, config.replay_gain);
 
-	listen_global_init(instance->event_loop, *instance->partition);
+	listen_global_init(raw_config, *instance->partitions.front().listener);
 
 #ifdef ENABLE_DAEMON
 	daemonize_set_user();
 	daemonize_begin(options.daemon);
+	AtScopeExit() { daemonize_finish(); };
 #endif
 
-	return mpd_main_after_fork(config);
+	return mpd_main_after_fork(raw_config, config);
+}
 
-} catch (const std::exception &e) {
-	LogError(e);
-	return EXIT_FAILURE;
+#ifdef ANDROID
+static inline
+#endif
+int mpd_main(int argc, char *argv[]) noexcept
+{
+	AtScopeExit() { log_deinit(); };
+
+	try {
+		return MainOrThrow(argc, argv);
+	} catch (...) {
+		LogError(std::current_exception());
+		return EXIT_FAILURE;
+	}
 }
 
 static int
-mpd_main_after_fork(const Config &config)
-try {
-	ConfigureFS();
+mpd_main_after_fork(const ConfigData &raw_config, const Config &config)
+{
+	ConfigureFS(raw_config);
 
-	glue_mapper_init();
+	glue_mapper_init(raw_config);
 
-	initPermissions();
-	spl_global_init();
+	initPermissions(raw_config);
+	spl_global_init(raw_config);
 #ifdef ENABLE_ARCHIVE
 	archive_plugin_init_all();
 #endif
 
-	pcm_convert_global_init();
+	pcm_convert_global_init(raw_config);
 
-	decoder_plugin_init_all();
+	decoder_plugin_init_all(raw_config);
 
 #ifdef ENABLE_DATABASE
-	const bool create_db = InitDatabaseAndStorage();
+	const bool create_db = InitDatabaseAndStorage(raw_config);
 #endif
 
-	glue_sticker_init();
+	glue_sticker_init(raw_config);
 
 	command_init();
 
-	instance->partition->outputs.Configure(instance->event_loop,
-					       config.replay_gain,
-					       instance->partition->pc);
-	instance->partition->UpdateEffectiveReplayGainMode();
+	for (auto &partition : instance->partitions) {
+		partition.outputs.Configure(instance->rtio_thread.GetEventLoop(),
+					    raw_config,
+					    config.replay_gain,
+					    partition.pc);
+		partition.UpdateEffectiveReplayGainMode();
+	}
 
-	client_manager_init();
-	input_stream_global_init();
-	playlist_list_global_init();
+	client_manager_init(raw_config);
+	input_stream_global_init(raw_config,
+				 instance->io_thread.GetEventLoop());
+	playlist_list_global_init(raw_config);
 
 #ifdef ENABLE_DAEMON
 	daemonize_commit();
@@ -532,39 +567,36 @@ try {
 	SignalHandlersInit(instance->event_loop);
 #endif
 
-	io_thread_start();
+	instance->io_thread.Start();
+	instance->rtio_thread.Start();
 
 #ifdef ENABLE_NEIGHBOR_PLUGINS
 	if (instance->neighbors != nullptr)
 		instance->neighbors->Open();
 #endif
 
-	ZeroconfInit(instance->event_loop);
-
-	StartPlayerThread(instance->partition->pc);
+	ZeroconfInit(raw_config, instance->event_loop);
 
 #ifdef ENABLE_DATABASE
 	if (create_db) {
 		/* the database failed to load: recreate the
 		   database */
-		unsigned job = instance->update->Enqueue("", true);
-		if (job == 0)
-			FatalError("directory update failed");
+		instance->update->Enqueue("", true);
 	}
 #endif
 
-	glue_state_file_init();
+	glue_state_file_init(raw_config);
 
 #ifdef ENABLE_DATABASE
-	if (config_get_bool(ConfigOption::AUTO_UPDATE, false)) {
+	if (raw_config.GetBool(ConfigOption::AUTO_UPDATE, false)) {
 #ifdef ENABLE_INOTIFY
 		if (instance->storage != nullptr &&
 		    instance->update != nullptr)
 			mpd_inotify_init(instance->event_loop,
 					 *instance->storage,
 					 *instance->update,
-					 config_get_unsigned(ConfigOption::AUTO_UPDATE_DEPTH,
-							     INT_MAX));
+					 raw_config.GetUnsigned(ConfigOption::AUTO_UPDATE_DEPTH,
+								INT_MAX));
 #else
 		FormatWarning(config_domain,
 			      "inotify: auto_update was disabled. enable during compilation phase");
@@ -572,11 +604,12 @@ try {
 	}
 #endif
 
-	config_global_check();
+	Check(raw_config);
 
 	/* enable all audio outputs (if not already done by
 	   playlist_state_restore() */
-	instance->partition->pc.LockUpdateAudio();
+	for (auto &partition : instance->partitions)
+		partition.pc.LockUpdateAudio();
 
 #ifdef _WIN32
 	win32_app_started();
@@ -599,21 +632,17 @@ try {
 
 	/* cleanup */
 
-#if defined(ENABLE_DATABASE) && defined(ENABLE_INOTIFY)
-	mpd_inotify_finish();
-
-	if (instance->update != nullptr)
-		instance->update->CancelAllAsync();
-#endif
+	instance->BeginShutdownUpdate();
 
 	if (instance->state_file != nullptr) {
 		instance->state_file->Write();
 		delete instance->state_file;
 	}
 
-	instance->partition->pc.Kill();
 	ZeroconfDeinit();
-	listen_global_finish();
+
+	instance->BeginShutdownPartitions();
+
 	delete instance->client_list;
 
 #ifdef ENABLE_NEIGHBOR_PLUGINS
@@ -623,16 +652,8 @@ try {
 	}
 #endif
 
-#ifdef ENABLE_DATABASE
-	delete instance->update;
-
-	if (instance->database != nullptr) {
-		instance->database->Close();
-		delete instance->database;
-	}
-
-	delete instance->storage;
-#endif
+	instance->FinishShutdownUpdate();
+	instance->ShutdownDatabase();
 
 #ifdef ENABLE_SQLITE
 	sticker_global_finish();
@@ -647,31 +668,19 @@ try {
 
 	DeinitFS();
 
-	delete instance->partition;
+	instance->FinishShutdownPartitions();
 	command_finish();
 	decoder_plugin_deinit_all();
 #ifdef ENABLE_ARCHIVE
 	archive_plugin_deinit_all();
 #endif
-	config_global_finish();
-	io_thread_deinit();
+	instance->rtio_thread.Stop();
+	instance->io_thread.Stop();
 #ifndef ANDROID
 	SignalHandlersFinish();
 #endif
-	delete instance;
-	instance = nullptr;
 
-#ifdef ENABLE_DAEMON
-	daemonize_finish();
-#endif
-
-	IcuFinish();
-
-	log_deinit();
 	return EXIT_SUCCESS;
-} catch (const std::exception &e) {
-	LogError(e);
-	return EXIT_FAILURE;
 }
 
 #ifdef ANDROID
@@ -700,7 +709,7 @@ JNIEXPORT void JNICALL
 Java_org_musicpd_Bridge_shutdown(JNIEnv *, jclass)
 {
 	if (instance != nullptr)
-		instance->Shutdown();
+		instance->Break();
 }
 
 #endif
