@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,14 +24,12 @@
 
 #include <assert.h>
 
-MusicChunk::~MusicChunk()
-{
-	delete tag;
-}
+MusicChunkInfo::MusicChunkInfo() noexcept = default;
+MusicChunkInfo::~MusicChunkInfo() noexcept = default;
 
 #ifndef NDEBUG
 bool
-MusicChunk::CheckFormat(const AudioFormat other_format) const
+MusicChunkInfo::CheckFormat(const AudioFormat other_format) const noexcept
 {
 	assert(other_format.IsValid());
 
@@ -41,7 +39,7 @@ MusicChunk::CheckFormat(const AudioFormat other_format) const
 
 WritableBuffer<void>
 MusicChunk::Write(const AudioFormat af,
-		  SongTime data_time, uint16_t _bit_rate)
+		  SongTime data_time, uint16_t _bit_rate) noexcept
 {
 	assert(CheckFormat(af));
 	assert(length == 0 || audio_format.IsValid());
@@ -64,7 +62,7 @@ MusicChunk::Write(const AudioFormat af,
 }
 
 bool
-MusicChunk::Expand(const AudioFormat af, size_t _length)
+MusicChunk::Expand(const AudioFormat af, size_t _length) noexcept
 {
 	const size_t frame_size = af.GetFrameSize();
 
