@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,14 +19,16 @@
 
 #include "config.h"
 #include "TagFile.hxx"
+#include "tag/Generic.hxx"
+#include "tag/Handler.hxx"
+#include "tag/Builder.hxx"
 #include "fs/Path.hxx"
-#include "util/UriUtil.hxx"
-#include "util/Error.hxx"
 #include "decoder/DecoderList.hxx"
 #include "decoder/DecoderPlugin.hxx"
 #include "input/InputStream.hxx"
 #include "input/LocalOpen.hxx"
-#include "thread/Cond.hxx"
+
+#include <exception>
 
 #include <assert.h>
 
@@ -34,65 +36,80 @@ class TagFileScan {
 	const Path path_fs;
 	const char *const suffix;
 
-	const tag_handler &handler;
-	void *handler_ctx;
+	TagHandler &handler;
 
 	Mutex mutex;
-	Cond cond;
-	InputStream *is;
+	InputStreamPtr is;
 
 public:
 	TagFileScan(Path _path_fs, const char *_suffix,
-		    const tag_handler &_handler, void *_handler_ctx)
+		    TagHandler &_handler) noexcept
 		:path_fs(_path_fs), suffix(_suffix),
-		 handler(_handler), handler_ctx(_handler_ctx) ,
+		 handler(_handler),
 		 is(nullptr) {}
 
-	~TagFileScan() {
-		delete is;
+	bool ScanFile(const DecoderPlugin &plugin) noexcept {
+		return plugin.ScanFile(path_fs, handler);
 	}
 
-	bool ScanFile(const DecoderPlugin &plugin) {
-		return plugin.ScanFile(path_fs, handler, handler_ctx);
-	}
-
-	bool ScanStream(const DecoderPlugin &plugin) {
+	bool ScanStream(const DecoderPlugin &plugin) noexcept {
 		if (plugin.scan_stream == nullptr)
 			return false;
 
 		/* open the InputStream (if not already open) */
 		if (is == nullptr) {
-			is = OpenLocalInputStream(path_fs,
-						  mutex, cond,
-						  IgnoreError());
-			if (is == nullptr)
+			try {
+				is = OpenLocalInputStream(path_fs, mutex);
+			} catch (...) {
 				return false;
-		} else
-			is->LockRewind(IgnoreError());
+			}
+		} else {
+			try {
+				is->LockRewind();
+			} catch (...) {
+			}
+		}
 
 		/* now try the stream_tag() method */
-		return plugin.ScanStream(*is, handler, handler_ctx);
+		return plugin.ScanStream(*is, handler);
 	}
 
-	bool Scan(const DecoderPlugin &plugin) {
+	bool Scan(const DecoderPlugin &plugin) noexcept {
 		return plugin.SupportsSuffix(suffix) &&
 			(ScanFile(plugin) || ScanStream(plugin));
 	}
 };
 
 bool
-tag_file_scan(Path path_fs, const tag_handler &handler, void *handler_ctx)
+ScanFileTagsNoGeneric(Path path_fs, TagHandler &handler) noexcept
 {
 	assert(!path_fs.IsNull());
 
 	/* check if there's a suffix and a plugin */
 
-	const char *suffix = uri_get_suffix(path_fs.c_str());
+	const auto *suffix = path_fs.GetSuffix();
 	if (suffix == nullptr)
 		return false;
 
-	TagFileScan tfs(path_fs, suffix, handler, handler_ctx);
+	const auto suffix_utf8 = Path::FromFS(suffix).ToUTF8();
+
+	TagFileScan tfs(path_fs, suffix_utf8.c_str(), handler);
 	return decoder_plugins_try([&](const DecoderPlugin &plugin){
 			return tfs.Scan(plugin);
 		});
 }
+
+bool
+ScanFileTagsWithGeneric(Path path, TagBuilder &builder,
+			AudioFormat *audio_format) noexcept
+{
+	FullTagHandler h(builder, audio_format);
+
+	if (!ScanFileTagsNoGeneric(path, h))
+		return false;
+
+	if (builder.empty())
+		ScanGenericTags(path, h);
+
+	return true;
+}
