@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,17 +22,17 @@
 #include "Log.hxx"
 
 void
-Client::SetExpired()
+Client::SetExpired() noexcept
 {
 	if (IsExpired())
 		return;
 
 	FullyBufferedSocket::Close();
-	TimeoutMonitor::Schedule(std::chrono::steady_clock::duration::zero());
+	timeout_event.Schedule(std::chrono::steady_clock::duration::zero());
 }
 
 void
-Client::OnTimeout()
+Client::OnTimeout() noexcept
 {
 	if (!IsExpired()) {
 		assert(!idle_waiting);
