@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,11 +20,13 @@
 #include "config.h"
 #include "OutputCommands.hxx"
 #include "Request.hxx"
-#include "output/OutputPrint.hxx"
+#include "output/Print.hxx"
 #include "output/OutputCommand.hxx"
 #include "client/Client.hxx"
 #include "client/Response.hxx"
 #include "Partition.hxx"
+#include "IdleFlags.hxx"
+#include "util/CharUtil.hxx"
 
 CommandResult
 handle_enableoutput(Client &client, Request args, Response &r)
@@ -32,7 +34,7 @@ handle_enableoutput(Client &client, Request args, Response &r)
 	assert(args.size == 1);
 	unsigned device = args.ParseUnsigned(0);
 
-	if (!audio_output_enable_index(client.partition.outputs, device)) {
+	if (!audio_output_enable_index(client.GetPartition().outputs, device)) {
 		r.Error(ACK_ERROR_NO_EXIST, "No such audio output");
 		return CommandResult::ERROR;
 	}
@@ -46,7 +48,7 @@ handle_disableoutput(Client &client, Request args, Response &r)
 	assert(args.size == 1);
 	unsigned device = args.ParseUnsigned(0);
 
-	if (!audio_output_disable_index(client.partition.outputs, device)) {
+	if (!audio_output_disable_index(client.GetPartition().outputs, device)) {
 		r.Error(ACK_ERROR_NO_EXIST, "No such audio output");
 		return CommandResult::ERROR;
 	}
@@ -60,7 +62,7 @@ handle_toggleoutput(Client &client, Request args, Response &r)
 	assert(args.size == 1);
 	unsigned device = args.ParseUnsigned(0);
 
-	if (!audio_output_toggle_index(client.partition.outputs, device)) {
+	if (!audio_output_toggle_index(client.GetPartition().outputs, device)) {
 		r.Error(ACK_ERROR_NO_EXIST, "No such audio output");
 		return CommandResult::ERROR;
 	}
@@ -68,11 +70,59 @@ handle_toggleoutput(Client &client, Request args, Response &r)
 	return CommandResult::OK;
 }
 
+static bool
+IsValidAttributeNameChar(char ch) noexcept
+{
+	return IsAlphaNumericASCII(ch) || ch == '_';
+}
+
+gcc_pure
+static bool
+IsValidAttributeName(const char *s) noexcept
+{
+	do {
+		if (!IsValidAttributeNameChar(*s))
+			return false;
+	} while (*++s);
+
+	return true;
+}
+
+CommandResult
+handle_outputset(Client &client, Request request, Response &response)
+{
+	assert(request.size == 3);
+	const unsigned i = request.ParseUnsigned(0);
+
+	auto &partition = client.GetPartition();
+	auto &outputs = partition.outputs;
+	if (i >= outputs.Size()) {
+		response.Error(ACK_ERROR_NO_EXIST, "No such audio output");
+		return CommandResult::ERROR;
+	}
+
+	auto &ao = outputs.Get(i);
+
+	const char *const name = request[1];
+	if (!IsValidAttributeName(name)) {
+		response.Error(ACK_ERROR_ARG, "Illegal attribute name");
+		return CommandResult::ERROR;
+	}
+
+	const char *const value = request[2];
+
+	ao.SetAttribute(name, value);
+
+	partition.EmitIdle(IDLE_OUTPUT);
+
+	return CommandResult::OK;
+}
+
 CommandResult
 handle_devices(Client &client, gcc_unused Request args, Response &r)
 {
-	assert(args.IsEmpty());
+	assert(args.empty());
 
-	printAudioDevices(r, client.partition.outputs);
+	printAudioDevices(r, client.GetPartition().outputs);
 	return CommandResult::OK;
 }
