@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,22 +20,28 @@
 #include "config.h"
 #include "DatabasePrint.hxx"
 #include "Selection.hxx"
-#include "SongFilter.hxx"
 #include "SongPrint.hxx"
 #include "TimePrint.hxx"
+#include "TagPrint.hxx"
 #include "client/Response.hxx"
+#include "protocol/RangeArg.hxx"
 #include "Partition.hxx"
+#include "song/DetachedSong.hxx"
+#include "song/Filter.hxx"
+#include "song/LightSong.hxx"
 #include "tag/Tag.hxx"
-#include "LightSong.hxx"
+#include "tag/Mask.hxx"
 #include "LightDirectory.hxx"
 #include "PlaylistInfo.hxx"
 #include "Interface.hxx"
 #include "fs/Traits.hxx"
+#include "util/ChronoUtil.hxx"
 
 #include <functional>
 
+gcc_pure
 static const char *
-ApplyBaseFlag(const char *uri, bool base)
+ApplyBaseFlag(const char *uri, bool base) noexcept
 {
 	if (base)
 		uri = PathTraitsUTF8::GetBase(uri);
@@ -43,26 +49,29 @@ ApplyBaseFlag(const char *uri, bool base)
 }
 
 static void
-PrintDirectoryURI(Response &r, bool base, const LightDirectory &directory)
+PrintDirectoryURI(Response &r, bool base,
+		  const LightDirectory &directory) noexcept
 {
 	r.Format("directory: %s\n",
 		 ApplyBaseFlag(directory.GetPath(), base));
 }
 
 static void
-PrintDirectoryBrief(Response &r, bool base, const LightDirectory &directory)
+PrintDirectoryBrief(Response &r, bool base,
+		    const LightDirectory &directory) noexcept
 {
 	if (!directory.IsRoot())
 		PrintDirectoryURI(r, base, directory);
 }
 
 static void
-PrintDirectoryFull(Response &r, bool base, const LightDirectory &directory)
+PrintDirectoryFull(Response &r, bool base,
+		   const LightDirectory &directory) noexcept
 {
 	if (!directory.IsRoot()) {
 		PrintDirectoryURI(r, base, directory);
 
-		if (directory.mtime > 0)
+		if (!IsNegative(directory.mtime))
 			time_print(r, "Last-Modified", directory.mtime);
 	}
 }
@@ -70,7 +79,7 @@ PrintDirectoryFull(Response &r, bool base, const LightDirectory &directory)
 static void
 print_playlist_in_directory(Response &r, bool base,
 			    const char *directory,
-			    const char *name_utf8)
+			    const char *name_utf8) noexcept
 {
 	if (base || directory == nullptr)
 		r.Format("playlist: %s\n",
@@ -83,7 +92,7 @@ print_playlist_in_directory(Response &r, bool base,
 static void
 print_playlist_in_directory(Response &r, bool base,
 			    const LightDirectory *directory,
-			    const char *name_utf8)
+			    const char *name_utf8) noexcept
 {
 	if (base || directory == nullptr || directory->IsRoot())
 		r.Format("playlist: %s\n", name_utf8);
@@ -93,24 +102,22 @@ print_playlist_in_directory(Response &r, bool base,
 }
 
 static void
-PrintSongBrief(Response &r, Partition &partition,
-	       bool base, const LightSong &song)
+PrintSongBrief(Response &r, bool base, const LightSong &song) noexcept
 {
-	song_print_uri(r, partition, song, base);
+	song_print_uri(r, song, base);
 
-	if (song.tag->has_playlist)
+	if (song.tag.has_playlist)
 		/* this song file has an embedded CUE sheet */
 		print_playlist_in_directory(r, base,
 					    song.directory, song.uri);
 }
 
 static void
-PrintSongFull(Response &r, Partition &partition,
-	      bool base, const LightSong &song)
+PrintSongFull(Response &r, bool base, const LightSong &song) noexcept
 {
-	song_print_info(r, partition, song, base);
+	song_print_info(r, song, base);
 
-	if (song.tag->has_playlist)
+	if (song.tag.has_playlist)
 		/* this song file has an embedded CUE sheet */
 		print_playlist_in_directory(r, base,
 					    song.directory, song.uri);
@@ -119,7 +126,7 @@ PrintSongFull(Response &r, Partition &partition,
 static void
 PrintPlaylistBrief(Response &r, bool base,
 		   const PlaylistInfo &playlist,
-		   const LightDirectory &directory)
+		   const LightDirectory &directory) noexcept
 {
 	print_playlist_in_directory(r, base,
 				    &directory, playlist.name.c_str());
@@ -128,62 +135,55 @@ PrintPlaylistBrief(Response &r, bool base,
 static void
 PrintPlaylistFull(Response &r, bool base,
 		  const PlaylistInfo &playlist,
-		  const LightDirectory &directory)
+		  const LightDirectory &directory) noexcept
 {
 	print_playlist_in_directory(r, base,
 				    &directory, playlist.name.c_str());
 
-	if (playlist.mtime > 0)
+	if (!IsNegative(playlist.mtime))
 		time_print(r, "Last-Modified", playlist.mtime);
 }
 
-void
-db_selection_print(Response &r, Partition &partition,
-		   const DatabaseSelection &selection,
-		   bool full, bool base,
-		   unsigned window_start, unsigned window_end)
-{
-	const Database &db = partition.GetDatabaseOrThrow();
-
-	unsigned i = 0;
-
-	using namespace std::placeholders;
-	const auto d = selection.filter == nullptr
-		? std::bind(full ? PrintDirectoryFull : PrintDirectoryBrief,
-			    std::ref(r), base, _1)
-		: VisitDirectory();
-	VisitSong s = std::bind(full ? PrintSongFull : PrintSongBrief,
-				std::ref(r), std::ref(partition), base, _1);
-	const auto p = selection.filter == nullptr
-		? std::bind(full ? PrintPlaylistFull : PrintPlaylistBrief,
-			    std::ref(r), base, _1, _2)
-		: VisitPlaylist();
-
-	if (window_start > 0 ||
-	    window_end < (unsigned)std::numeric_limits<int>::max())
-		s = [s, window_start, window_end, &i](const LightSong &song){
-			const bool in_window = i >= window_start && i < window_end;
-			++i;
-			if (in_window)
-				s(song);
-		};
-
-	db.Visit(selection, d, s, p);
-}
-
 void
 db_selection_print(Response &r, Partition &partition,
 		   const DatabaseSelection &selection,
 		   bool full, bool base)
 {
-	db_selection_print(r, partition, selection, full, base,
-			   0, std::numeric_limits<int>::max());
+	const Database &db = partition.GetDatabaseOrThrow();
+
+	using namespace std::placeholders;
+	const auto d = selection.filter == nullptr
+		? std::bind(full ? PrintDirectoryFull : PrintDirectoryBrief,
+			    std::ref(r), base, _1)
+		: VisitDirectory();
+	VisitSong s = std::bind(full ? PrintSongFull : PrintSongBrief,
+				std::ref(r), base, _1);
+	const auto p = selection.filter == nullptr
+		? std::bind(full ? PrintPlaylistFull : PrintPlaylistBrief,
+			    std::ref(r), base, _1, _2)
+		: VisitPlaylist();
+
+	db.Visit(selection, d, s, p);
 }
 
 static void
-PrintSongURIVisitor(Response &r, Partition &partition, const LightSong &song)
+PrintSongURIVisitor(Response &r, const LightSong &song) noexcept
 {
-	song_print_uri(r, partition, song);
+	song_print_uri(r, song);
+}
+
+void
+PrintSongUris(Response &r, Partition &partition,
+	      const SongFilter *filter)
+{
+	const Database &db = partition.GetDatabaseOrThrow();
+
+	const DatabaseSelection selection("", true, filter);
+
+	using namespace std::placeholders;
+	const auto f = std::bind(PrintSongURIVisitor,
+				 std::ref(r), _1);
+	db.Visit(selection, f);
 }
 
 static void
@@ -214,24 +214,15 @@ PrintGroupedUniqueTags(Response &r, TagType tag_type, TagType group,
 
 void
 PrintUniqueTags(Response &r, Partition &partition,
-		unsigned type, TagType group,
+		TagType type, TagType group,
 		const SongFilter *filter)
 {
+	assert(type < TAG_NUM_OF_ITEM_TYPES);
+
 	const Database &db = partition.GetDatabaseOrThrow();
 
 	const DatabaseSelection selection("", true, filter);
 
-	if (type == LOCATE_TAG_FILE_TYPE) {
-		using namespace std::placeholders;
-		const auto f = std::bind(PrintSongURIVisitor,
-					 std::ref(r), std::ref(partition), _1);
-		db.Visit(selection, f);
-	} else {
-		assert(type < TAG_NUM_OF_ITEM_TYPES);
-
-		PrintGroupedUniqueTags(r, TagType(type), group,
-				       db.CollectUniqueTags(selection,
-							    TagType(type),
-							    group));
-	}
+	PrintGroupedUniqueTags(r, type, group,
+			       db.CollectUniqueTags(selection, type, group));
 }
