@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
 
 #include "UniqueTags.hxx"
 #include "Interface.hxx"
-#include "LightSong.hxx"
+#include "song/LightSong.hxx"
 #include "tag/VisitFallback.hxx"
 
 #include <functional>
@@ -54,7 +54,7 @@ CollectUniqueTags(const Database &db, const DatabaseSelection &selection,
 	std::map<std::string, std::set<std::string>> result;
 
 	db.Visit(selection, [&result, tag_type, group](const LightSong &song){
-			CollectGroupTags(result, *song.tag, tag_type, group);
+			CollectGroupTags(result, song.tag, tag_type, group);
 		});
 
 	return result;
