@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,19 +24,19 @@
 #include <assert.h>
 
 bool
-DecoderPlugin::SupportsSuffix(const char *suffix) const
+DecoderPlugin::SupportsSuffix(const char *suffix) const noexcept
 {
 #if !CLANG_CHECK_VERSION(3,6)
 	/* disabled on clang due to -Wtautological-pointer-compare */
 	assert(suffix != nullptr);
 #endif
 
-	return suffixes != nullptr && string_array_contains(suffixes, suffix);
-
+	return suffixes != nullptr &&
+		StringArrayContainsCase(suffixes, suffix);
 }
 
 bool
-DecoderPlugin::SupportsMimeType(const char *mime_type) const
+DecoderPlugin::SupportsMimeType(const char *mime_type) const noexcept
 {
 #if !CLANG_CHECK_VERSION(3,6)
 	/* disabled on clang due to -Wtautological-pointer-compare */
@@ -44,5 +44,5 @@ DecoderPlugin::SupportsMimeType(const char *mime_type) const
 #endif
 
 	return mime_types != nullptr &&
-		string_array_contains(mime_types, mime_type);
+		StringArrayContainsCase(mime_types, mime_type);
 }
