@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,8 +32,8 @@
 #include "FfmpegMetaData.hxx"
 #include "FfmpegIo.hxx"
 #include "pcm/Interleave.hxx"
-#include "tag/TagBuilder.hxx"
-#include "tag/TagHandler.hxx"
+#include "tag/Builder.hxx"
+#include "tag/Handler.hxx"
 #include "tag/ReplayGain.hxx"
 #include "tag/MixRamp.hxx"
 #include "input/InputStream.hxx"
@@ -47,10 +47,7 @@ extern "C" {
 #include <libavformat/avformat.h>
 #include <libavformat/avio.h>
 #include <libavutil/avutil.h>
-
-#if LIBAVUTIL_VERSION_MAJOR >= 53
 #include <libavutil/frame.h>
-#endif
 }
 
 #include <assert.h>
@@ -103,7 +100,7 @@ ffmpeg_init(const ConfigBlock &block)
 }
 
 static void
-ffmpeg_finish()
+ffmpeg_finish() noexcept
 {
 	av_dict_free(&avformat_options);
 }
@@ -258,9 +255,9 @@ FfmpegSendFrame(DecoderClient &client, InputStream &is,
 	try {
 		output_buffer = copy_interleave_frame(codec_context, frame,
 						      buffer);
-	} catch (const std::exception e) {
+	} catch (...) {
 		/* this must be a serious error, e.g. OOM */
-		LogError(e);
+		LogError(std::current_exception());
 		return DecoderCommand::STOP;
 	}
 
@@ -470,55 +467,6 @@ ffmpeg_sample_format(enum AVSampleFormat sample_fmt) noexcept
 	return SampleFormat::UNDEFINED;
 }
 
-static AVInputFormat *
-ffmpeg_probe(DecoderClient *client, InputStream &is)
-{
-	constexpr size_t BUFFER_SIZE = 16384;
-	constexpr size_t PADDING = 16;
-
-	unsigned char buffer[BUFFER_SIZE];
-	size_t nbytes = decoder_read(client, is, buffer, BUFFER_SIZE);
-	if (nbytes <= PADDING)
-		return nullptr;
-
-	try {
-		is.LockRewind();
-	} catch (const std::runtime_error &) {
-		return nullptr;
-	}
-
-	/* some ffmpeg parsers (e.g. ac3_parser.c) read a few bytes
-	   beyond the declared buffer limit, which makes valgrind
-	   angry; this workaround removes some padding from the buffer
-	   size */
-	nbytes -= PADDING;
-
-	AVProbeData avpd;
-
-	/* new versions of ffmpeg may add new attributes, and leaving
-	   them uninitialized may crash; hopefully, zero-initializing
-	   everything we don't know is ok */
-	memset(&avpd, 0, sizeof(avpd));
-
-	avpd.buf = buffer;
-	avpd.buf_size = nbytes;
-	avpd.filename = is.GetURI();
-
-#ifdef AVPROBE_SCORE_MIME
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(56, 5, 1)
-	/* this attribute was added in libav/ffmpeg version 11, but
-	   unfortunately it's "uint8_t" instead of "char", and it's
-	   not "const" - wtf? */
-	avpd.mime_type = (uint8_t *)const_cast<char *>(is.GetMimeType());
-#else
-	/* API problem fixed in FFmpeg 2.5 */
-	avpd.mime_type = is.GetMimeType();
-#endif
-#endif
-
-	return av_probe_input_format(&avpd, true);
-}
-
 static void
 FfmpegParseMetaData(AVDictionary &dict, ReplayGainInfo &rg, MixRampInfo &mr)
 {
@@ -572,31 +520,28 @@ FfmpegParseMetaData(DecoderClient &client,
 }
 
 static void
-FfmpegScanMetadata(const AVStream &stream,
-		   const TagHandler &handler, void *handler_ctx)
+FfmpegScanMetadata(const AVStream &stream, TagHandler &handler) noexcept
 {
-	FfmpegScanDictionary(stream.metadata, handler, handler_ctx);
+	FfmpegScanDictionary(stream.metadata, handler);
 }
 
 static void
 FfmpegScanMetadata(const AVFormatContext &format_context, int audio_stream,
-		   const TagHandler &handler, void *handler_ctx)
+		   TagHandler &handler) noexcept
 {
 	assert(audio_stream >= 0);
 
-	FfmpegScanDictionary(format_context.metadata, handler, handler_ctx);
+	FfmpegScanDictionary(format_context.metadata, handler);
 	FfmpegScanMetadata(*format_context.streams[audio_stream],
-			   handler, handler_ctx);
+			   handler);
 }
 
-#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(56, 1, 0)
-
 static void
 FfmpegScanTag(const AVFormatContext &format_context, int audio_stream,
 	      TagBuilder &tag)
 {
-	FfmpegScanMetadata(format_context, audio_stream,
-			   full_tag_handler, &tag);
+	FullTagHandler h(tag);
+	FfmpegScanMetadata(format_context, audio_stream, h);
 }
 
 /**
@@ -617,12 +562,10 @@ FfmpegCheckTag(DecoderClient &client, InputStream &is,
 
 	TagBuilder tag;
 	FfmpegScanTag(format_context, audio_stream, tag);
-	if (!tag.IsEmpty())
+	if (!tag.empty())
 		client.SubmitTag(is, tag.Commit());
 }
 
-#endif
-
 static void
 FfmpegDecode(DecoderClient &client, InputStream &input,
 	     AVFormatContext &format_context)
@@ -648,17 +591,11 @@ FfmpegDecode(DecoderClient &client, InputStream &input,
 
 	const auto &codec_params = GetCodecParameters(av_stream);
 
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(54, 25, 0)
 	const AVCodecDescriptor *codec_descriptor =
 		avcodec_descriptor_get(codec_params.codec_id);
 	if (codec_descriptor != nullptr)
 		FormatDebug(ffmpeg_domain, "codec '%s'",
 			    codec_descriptor->name);
-#else
-	if (codec_context->codec_name[0] != 0)
-		FormatDebug(ffmpeg_domain, "codec '%s'",
-			    codec_context->codec_name);
-#endif
 
 	AVCodec *codec = avcodec_find_decoder(codec_params.codec_id);
 
@@ -712,30 +649,22 @@ FfmpegDecode(DecoderClient &client, InputStream &input,
 #endif
 
 	const SignedSongTime total_time =
-		FromFfmpegTimeChecked(av_stream.duration, av_stream.time_base);
+		av_stream.duration != (int64_t)AV_NOPTS_VALUE
+		? FromFfmpegTimeChecked(av_stream.duration, av_stream.time_base)
+		: FromFfmpegTimeChecked(format_context.duration, AV_TIME_BASE_Q);
 
 	client.Ready(audio_format, input.IsSeekable(), total_time);
 
 	FfmpegParseMetaData(client, format_context, audio_stream);
 
-#if LIBAVUTIL_VERSION_MAJOR >= 53
 	AVFrame *frame = av_frame_alloc();
-#else
-	AVFrame *frame = avcodec_alloc_frame();
-#endif
 	if (!frame) {
 		LogError(ffmpeg_domain, "Could not allocate frame");
 		return;
 	}
 
 	AtScopeExit(&frame) {
-#if LIBAVUTIL_VERSION_MAJOR >= 53
 		av_frame_free(&frame);
-#elif LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(54, 28, 0)
-		avcodec_free_frame(&frame);
-#else
-		av_free(frame);
-#endif
 	};
 
 	FfmpegBuffer interleaved_buffer;
@@ -768,9 +697,7 @@ FfmpegDecode(DecoderClient &client, InputStream &input,
 			/* end of file */
 			break;
 
-#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(56, 1, 0)
 		FfmpegCheckTag(client, input, format_context, audio_stream);
-#endif
 
 		if (packet.size > 0 && packet.stream_index == audio_stream) {
 			cmd = ffmpeg_send_packet(client, input,
@@ -795,13 +722,6 @@ FfmpegDecode(DecoderClient &client, InputStream &input,
 static void
 ffmpeg_decode(DecoderClient &client, InputStream &input)
 {
-	AVInputFormat *input_format = ffmpeg_probe(&client, input);
-	if (input_format == nullptr)
-		return;
-
-	FormatDebug(ffmpeg_domain, "detected input format '%s' (%s)",
-		    input_format->name, input_format->long_name);
-
 	AvioStream stream(&client, input);
 	if (!stream.Open()) {
 		LogError(ffmpeg_domain, "Failed to open stream");
@@ -811,9 +731,9 @@ ffmpeg_decode(DecoderClient &client, InputStream &input)
 	AVFormatContext *format_context;
 	try {
 		format_context =FfmpegOpenInput(stream.io, input.GetURI(),
-						input_format);
-	} catch (const std::runtime_error &e) {
-		LogError(e);
+						nullptr);
+	} catch (...) {
+		LogError(std::current_exception());
 		return;
 	}
 
@@ -821,12 +741,16 @@ ffmpeg_decode(DecoderClient &client, InputStream &input)
 		avformat_close_input(&format_context);
 	};
 
+	const auto *input_format = format_context->iformat;
+	FormatDebug(ffmpeg_domain, "detected input format '%s' (%s)",
+		    input_format->name, input_format->long_name);
+
 	FfmpegDecode(client, input, *format_context);
 }
 
 static bool
 FfmpegScanStream(AVFormatContext &format_context,
-		 const TagHandler &handler, void *handler_ctx)
+		 TagHandler &handler) noexcept
 {
 	const int find_result =
 		avformat_find_stream_info(&format_context, nullptr);
@@ -839,31 +763,36 @@ FfmpegScanStream(AVFormatContext &format_context,
 
 	const AVStream &stream = *format_context.streams[audio_stream];
 	if (stream.duration != (int64_t)AV_NOPTS_VALUE)
-		tag_handler_invoke_duration(handler, handler_ctx,
-					    FromFfmpegTime(stream.duration,
-							   stream.time_base));
+		handler.OnDuration(FromFfmpegTime(stream.duration,
+						  stream.time_base));
+	else if (format_context.duration != (int64_t)AV_NOPTS_VALUE)
+		handler.OnDuration(FromFfmpegTime(format_context.duration,
+						  AV_TIME_BASE_Q));
 
-	FfmpegScanMetadata(format_context, audio_stream, handler, handler_ctx);
+	const auto &codec_params = GetCodecParameters(stream);
+	try {
+		handler.OnAudioFormat(CheckAudioFormat(codec_params.sample_rate,
+						       ffmpeg_sample_format(GetSampleFormat(codec_params)),
+						       codec_params.channels));
+	} catch (...) {
+	}
+
+	FfmpegScanMetadata(format_context, audio_stream, handler);
 
 	return true;
 }
 
 static bool
-ffmpeg_scan_stream(InputStream &is,
-		   const TagHandler &handler, void *handler_ctx)
+ffmpeg_scan_stream(InputStream &is, TagHandler &handler) noexcept
 {
-	AVInputFormat *input_format = ffmpeg_probe(nullptr, is);
-	if (input_format == nullptr)
-		return false;
-
 	AvioStream stream(nullptr, is);
 	if (!stream.Open())
 		return false;
 
 	AVFormatContext *f;
 	try {
-		f = FfmpegOpenInput(stream.io, is.GetURI(), input_format);
-	} catch (const std::runtime_error &) {
+		f = FfmpegOpenInput(stream.io, is.GetURI(), nullptr);
+	} catch (...) {
 		return false;
 	}
 
@@ -871,7 +800,7 @@ ffmpeg_scan_stream(InputStream &is,
 		avformat_close_input(&f);
 	};
 
-	return FfmpegScanStream(*f, handler, handler_ctx);
+	return FfmpegScanStream(*f, handler);
 }
 
 /**
