@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,9 +23,9 @@
 #include "../DecoderAPI.hxx"
 #include "input/InputStream.hxx"
 #include "Log.hxx"
-#include "Compiler.h"
+#include "util/Compiler.h"
 
-#include <stdexcept>
+#include <exception>
 
 FLAC__StreamDecoderReadStatus
 FlacInput::Read(FLAC__byte buffer[], size_t *bytes)
@@ -54,8 +54,8 @@ FlacInput::Seek(FLAC__uint64 absolute_byte_offset)
 	try {
 		input_stream.LockSeek(absolute_byte_offset);
 		return FLAC__STREAM_DECODER_SEEK_STATUS_OK;
-	} catch (const std::runtime_error &e) {
-		LogError(e);
+	} catch (...) {
+		LogError(std::current_exception());
 		return FLAC__STREAM_DECODER_SEEK_STATUS_ERROR;
 	}
 }
