@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,12 +21,13 @@
 #include "Mpg123DecoderPlugin.hxx"
 #include "../DecoderAPI.hxx"
 #include "CheckAudioFormat.hxx"
-#include "tag/TagHandler.hxx"
-#include "tag/TagBuilder.hxx"
+#include "tag/Handler.hxx"
+#include "tag/Builder.hxx"
 #include "tag/ReplayGain.hxx"
 #include "tag/MixRamp.hxx"
 #include "fs/Path.hxx"
 #include "util/Domain.hxx"
+#include "util/ScopeExit.hxx"
 #include "util/StringView.hxx"
 #include "Log.hxx"
 
@@ -45,7 +46,7 @@ mpd_mpg123_init(gcc_unused const ConfigBlock &block)
 }
 
 static void
-mpd_mpg123_finish(void)
+mpd_mpg123_finish() noexcept
 {
 	mpg123_exit();
 }
@@ -63,10 +64,7 @@ static bool
 mpd_mpg123_open(mpg123_handle *handle, const char *path_fs,
 		AudioFormat &audio_format)
 {
-	/* mpg123_open() wants a writable string :-( */
-	char *const path2 = const_cast<char *>(path_fs);
-
-	int error = mpg123_open(handle, path2);
+	int error = mpg123_open(handle, path_fs);
 	if (error != MPG123_OK) {
 		FormatWarning(mpg123_domain,
 			      "libmpg123 failed to open %s: %s",
@@ -196,11 +194,11 @@ mpd_mpg123_file_decode(DecoderClient &client, Path path_fs)
 		return;
 	}
 
+	AtScopeExit(handle) { mpg123_delete(handle); };
+
 	AudioFormat audio_format;
-	if (!mpd_mpg123_open(handle, path_fs.c_str(), audio_format)) {
-		mpg123_delete(handle);
+	if (!mpd_mpg123_open(handle, path_fs.c_str(), audio_format))
 		return;
-	}
 
 	const off_t num_samples = mpg123_length(handle);
 
@@ -266,21 +264,16 @@ mpd_mpg123_file_decode(DecoderClient &client, Path path_fs)
 				client.SeekError();
 			else {
 				client.CommandFinished();
-				client.SubmitTimestamp(c / (double)audio_format.sample_rate);
+				client.SubmitTimestamp(audio_format.FramesToTime<FloatDuration>(c));
 			}
 
 			cmd = DecoderCommand::NONE;
 		}
 	} while (cmd == DecoderCommand::NONE);
-
-	/* cleanup */
-
-	mpg123_delete(handle);
 }
 
 static bool
-mpd_mpg123_scan_file(Path path_fs,
-		     const TagHandler &handler, void *handler_ctx)
+mpd_mpg123_scan_file(Path path_fs, TagHandler &handler) noexcept
 {
 	int error;
 	mpg123_handle *const handle = mpg123_new(nullptr, &error);
@@ -291,27 +284,31 @@ mpd_mpg123_scan_file(Path path_fs,
 		return false;
 	}
 
+	AtScopeExit(handle) { mpg123_delete(handle); };
+
 	AudioFormat audio_format;
-	if (!mpd_mpg123_open(handle, path_fs.c_str(), audio_format)) {
-		mpg123_delete(handle);
+	try {
+		if (!mpd_mpg123_open(handle, path_fs.c_str(), audio_format)) {
+			return false;
+		}
+	} catch (...) {
 		return false;
 	}
 
 	const off_t num_samples = mpg123_length(handle);
 	if (num_samples <= 0) {
-		mpg123_delete(handle);
 		return false;
 	}
 
-	/* ID3 tag support not yet implemented */
+	handler.OnAudioFormat(audio_format);
 
-	mpg123_delete(handle);
+	/* ID3 tag support not yet implemented */
 
 	const auto duration =
 		SongTime::FromScale<uint64_t>(num_samples,
 					      audio_format.sample_rate);
 
-	tag_handler_invoke_duration(handler, handler_ctx, duration);
+	handler.OnDuration(duration);
 	return true;
 }
 
