@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,18 +20,19 @@
 #include "config.h"
 #include "Call.hxx"
 #include "Loop.hxx"
-#include "DeferredMonitor.hxx"
+#include "DeferEvent.hxx"
 #include "thread/Mutex.hxx"
 #include "thread/Cond.hxx"
-#include "Compiler.h"
+#include "util/Compiler.h"
 
 #include <exception>
 
 #include <assert.h>
 
 class BlockingCallMonitor final
-	: DeferredMonitor
 {
+	DeferEvent defer_event;
+
 	const std::function<void()> f;
 
 	Mutex mutex;
@@ -43,12 +44,13 @@ class BlockingCallMonitor final
 
 public:
 	BlockingCallMonitor(EventLoop &_loop, std::function<void()> &&_f)
-		:DeferredMonitor(_loop), f(std::move(_f)), done(false) {}
+		:defer_event(_loop, BIND_THIS_METHOD(RunDeferred)),
+		 f(std::move(_f)), done(false) {}
 
 	void Run() {
 		assert(!done);
 
-		Schedule();
+		defer_event.Schedule();
 
 		mutex.lock();
 		while (!done)
@@ -60,7 +62,7 @@ public:
 	}
 
 private:
-	virtual void RunDeferred() override {
+	void RunDeferred() noexcept {
 		assert(!done);
 
 		try {
@@ -79,7 +81,7 @@ private:
 void
 BlockingCall(EventLoop &loop, std::function<void()> &&f)
 {
-	if (loop.IsInsideOrNull()) {
+	if (loop.IsDead() || loop.IsInside()) {
 		/* we're already inside the loop - we can simply call
 		   the function */
 		f();
