@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,37 +24,32 @@
 #include <assert.h>
 
 void
-IdleMonitor::Cancel()
+IdleMonitor::Cancel() noexcept
 {
-	assert(loop.IsInsideOrNull());
+	assert(loop.IsInside());
 
 	if (!IsActive())
 		return;
 
-	active = false;
 	loop.RemoveIdle(*this);
 }
 
 void
-IdleMonitor::Schedule()
+IdleMonitor::Schedule() noexcept
 {
-	assert(loop.IsInsideOrVirgin());
+	assert(loop.IsInside());
 
 	if (IsActive())
 		/* already scheduled */
 		return;
 
-	active = true;
 	loop.AddIdle(*this);
 }
 
 void
-IdleMonitor::Run()
+IdleMonitor::Run() noexcept
 {
 	assert(loop.IsInside());
 
-	assert(active);
-	active = false;
-
 	OnIdle();
 }
