@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -78,7 +78,7 @@ GetParentPathImpl(typename Traits::const_pointer_type p)
 		return typename Traits::string(Traits::CURRENT_DIRECTORY);
 	if (sep == p)
 		return typename Traits::string(p, p + 1);
-#ifdef WIN32
+#ifdef _WIN32
 	if (Traits::IsDrive(p) && sep == p + 2)
 		return typename Traits::string(p, p + 3);
 #endif
@@ -99,9 +99,16 @@ RelativePathImpl(typename Traits::const_pointer_type base,
 		return nullptr;
 
 	if (*other != 0) {
-		if (!Traits::IsSeparator(*other))
+		if (!Traits::IsSeparator(*other)) {
+			if (*base != 0 && Traits::IsSeparator(other[-1]))
+				/* "other" has no more slash, but the
+				   matching base ended with a slash:
+				   enough to detect a match */
+				return other;
+
 			/* mismatch */
 			return nullptr;
+		}
 
 		/* skip remaining path separators */
 		do {
@@ -137,6 +144,20 @@ PathTraitsFS::Relative(const_pointer_type base, const_pointer_type other) noexce
 	return RelativePathImpl<PathTraitsFS>(base, other);
 }
 
+PathTraitsFS::string
+PathTraitsFS::Apply(const_pointer_type base, const_pointer_type path) noexcept
+{
+	// TODO: support the Windows syntax (absolute path with or without drive, drive with relative path)
+
+	if (base == nullptr)
+		return path;
+
+	if (IsAbsolute(path))
+		return path;
+
+	return Build(base, path);
+}
+
 PathTraitsUTF8::string
 PathTraitsUTF8::Build(const_pointer_type a, size_t a_size,
 		      const_pointer_type b, size_t b_size) noexcept
