@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -39,9 +39,9 @@ struct FfmpegInputStream final : public InputStream {
 
 	bool eof;
 
-	FfmpegInputStream(const char *_uri, Mutex &_mutex, Cond &_cond,
+	FfmpegInputStream(const char *_uri, Mutex &_mutex,
 			  AVIOContext *_h)
-		:InputStream(_uri, _mutex, _cond),
+		:InputStream(_uri, _mutex),
 		 h(_h), eof(false) {
 		seekable = (h->seekable & AVIO_SEEKABLE_NORMAL) != 0;
 		size = avio_size(h);
@@ -72,7 +72,7 @@ input_ffmpeg_supported(void)
 }
 
 static void
-input_ffmpeg_init(gcc_unused const ConfigBlock &block)
+input_ffmpeg_init(EventLoop &, const ConfigBlock &)
 {
 	FfmpegInit();
 
@@ -81,24 +81,16 @@ input_ffmpeg_init(gcc_unused const ConfigBlock &block)
 		throw PluginUnavailable("No protocol");
 }
 
-static InputStream *
+static InputStreamPtr
 input_ffmpeg_open(const char *uri,
-		  Mutex &mutex, Cond &cond)
+		  Mutex &mutex)
 {
-	if (!StringStartsWithCaseASCII(uri, "gopher://") &&
-	    !StringStartsWithCaseASCII(uri, "rtp://") &&
-	    !StringStartsWithCaseASCII(uri, "rtsp://") &&
-	    !StringStartsWithCaseASCII(uri, "rtmp://") &&
-	    !StringStartsWithCaseASCII(uri, "rtmpt://") &&
-	    !StringStartsWithCaseASCII(uri, "rtmps://"))
-		return nullptr;
-
 	AVIOContext *h;
 	auto result = avio_open(&h, uri, AVIO_FLAG_READ);
 	if (result != 0)
 		throw MakeFfmpegError(result);
 
-	return new FfmpegInputStream(uri, mutex, cond, h);
+	return std::make_unique<FfmpegInputStream>(uri, mutex, h);
 }
 
 size_t
@@ -146,8 +138,19 @@ FfmpegInputStream::Seek(offset_type new_offset)
 	eof = false;
 }
 
+static constexpr const char *ffmpeg_prefixes[] = {
+	"gopher://",
+	"rtp://",
+	"rtsp://",
+	"rtmp://",
+	"rtmpt://",
+	"rtmps://",
+	nullptr
+};
+
 const InputPlugin input_plugin_ffmpeg = {
 	"ffmpeg",
+	ffmpeg_prefixes,
 	input_ffmpeg_init,
 	nullptr,
 	input_ffmpeg_open,
