@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 #include "Lease.hxx"
 #include "Callback.hxx"
 #include "event/Loop.hxx"
-#include "system/fd_util.h"
+#include "net/SocketDescriptor.hxx"
 #include "util/RuntimeError.hxx"
 
 extern "C" {
@@ -53,6 +53,18 @@ NfsConnection::CancellableCallback::Stat(nfs_context *ctx,
 					 nfs_get_error(ctx));
 }
 
+inline void
+NfsConnection::CancellableCallback::Lstat(nfs_context *ctx,
+					  const char *path)
+{
+	assert(connection.GetEventLoop().IsInside());
+
+	int result = nfs_lstat64_async(ctx, path, Callback, this);
+	if (result < 0)
+		throw FormatRuntimeError("nfs_lstat64_async() failed: %s",
+					 nfs_get_error(ctx));
+}
+
 inline void
 NfsConnection::CancellableCallback::OpenDirectory(nfs_context *ctx,
 						  const char *path)
@@ -103,7 +115,7 @@ NfsConnection::CancellableCallback::Read(nfs_context *ctx, struct nfsfh *fh,
 }
 
 inline void
-NfsConnection::CancellableCallback::CancelAndScheduleClose(struct nfsfh *fh)
+NfsConnection::CancellableCallback::CancelAndScheduleClose(struct nfsfh *fh) noexcept
 {
 	assert(connection.GetEventLoop().IsInside());
 	assert(!open);
@@ -115,7 +127,7 @@ NfsConnection::CancellableCallback::CancelAndScheduleClose(struct nfsfh *fh)
 }
 
 inline void
-NfsConnection::CancellableCallback::PrepareDestroyContext()
+NfsConnection::CancellableCallback::PrepareDestroyContext() noexcept
 {
 	assert(IsCancelled());
 
@@ -126,7 +138,7 @@ NfsConnection::CancellableCallback::PrepareDestroyContext()
 }
 
 inline void
-NfsConnection::CancellableCallback::Callback(int err, void *data)
+NfsConnection::CancellableCallback::Callback(int err, void *data) noexcept
 {
 	assert(connection.GetEventLoop().IsInside());
 
@@ -162,27 +174,28 @@ NfsConnection::CancellableCallback::Callback(int err, void *data)
 void
 NfsConnection::CancellableCallback::Callback(int err,
 					     gcc_unused struct nfs_context *nfs,
-					     void *data, void *private_data)
+					     void *data,
+					     void *private_data) noexcept
 {
 	CancellableCallback &c = *(CancellableCallback *)private_data;
 	c.Callback(err, data);
 }
 
 static constexpr unsigned
-libnfs_to_events(int i)
+libnfs_to_events(int i) noexcept
 {
 	return ((i & POLLIN) ? SocketMonitor::READ : 0) |
 		((i & POLLOUT) ? SocketMonitor::WRITE : 0);
 }
 
 static constexpr int
-events_to_libnfs(unsigned i)
+events_to_libnfs(unsigned i) noexcept
 {
 	return ((i & SocketMonitor::READ) ? POLLIN : 0) |
 		((i & SocketMonitor::WRITE) ? POLLOUT : 0);
 }
 
-NfsConnection::~NfsConnection()
+NfsConnection::~NfsConnection() noexcept
 {
 	assert(GetEventLoop().IsInside());
 	assert(new_leases.empty());
@@ -195,17 +208,17 @@ NfsConnection::~NfsConnection()
 }
 
 void
-NfsConnection::AddLease(NfsLease &lease)
+NfsConnection::AddLease(NfsLease &lease) noexcept
 {
 	assert(GetEventLoop().IsInside());
 
 	new_leases.push_back(&lease);
 
-	DeferredMonitor::Schedule();
+	defer_new_lease.Schedule();
 }
 
 void
-NfsConnection::RemoveLease(NfsLease &lease)
+NfsConnection::RemoveLease(NfsLease &lease) noexcept
 {
 	assert(GetEventLoop().IsInside());
 
@@ -230,6 +243,23 @@ NfsConnection::Stat(const char *path, NfsCallback &callback)
 	ScheduleSocket();
 }
 
+void
+NfsConnection::Lstat(const char *path, NfsCallback &callback)
+{
+	assert(GetEventLoop().IsInside());
+	assert(!callbacks.Contains(callback));
+
+	auto &c = callbacks.Add(callback, *this, false);
+	try {
+		c.Lstat(context, path);
+	} catch (...) {
+		callbacks.Remove(c);
+		throw;
+	}
+
+	ScheduleSocket();
+}
+
 void
 NfsConnection::OpenDirectory(const char *path, NfsCallback &callback)
 {
@@ -248,7 +278,7 @@ NfsConnection::OpenDirectory(const char *path, NfsCallback &callback)
 }
 
 const struct nfsdirent *
-NfsConnection::ReadDirectory(struct nfsdir *dir)
+NfsConnection::ReadDirectory(struct nfsdir *dir) noexcept
 {
 	assert(GetEventLoop().IsInside());
 
@@ -256,7 +286,7 @@ NfsConnection::ReadDirectory(struct nfsdir *dir)
 }
 
 void
-NfsConnection::CloseDirectory(struct nfsdir *dir)
+NfsConnection::CloseDirectory(struct nfsdir *dir) noexcept
 {
 	assert(GetEventLoop().IsInside());
 
@@ -316,18 +346,18 @@ NfsConnection::Read(struct nfsfh *fh, uint64_t offset, size_t size,
 }
 
 void
-NfsConnection::Cancel(NfsCallback &callback)
+NfsConnection::Cancel(NfsCallback &callback) noexcept
 {
 	callbacks.Cancel(callback);
 }
 
 static void
-DummyCallback(int, struct nfs_context *, void *, void *)
+DummyCallback(int, struct nfs_context *, void *, void *) noexcept
 {
 }
 
 inline void
-NfsConnection::InternalClose(struct nfsfh *fh)
+NfsConnection::InternalClose(struct nfsfh *fh) noexcept
 {
 	assert(GetEventLoop().IsInside());
 	assert(context != nullptr);
@@ -337,7 +367,7 @@ NfsConnection::InternalClose(struct nfsfh *fh)
 }
 
 void
-NfsConnection::Close(struct nfsfh *fh)
+NfsConnection::Close(struct nfsfh *fh) noexcept
 {
 	assert(GetEventLoop().IsInside());
 
@@ -346,14 +376,14 @@ NfsConnection::Close(struct nfsfh *fh)
 }
 
 void
-NfsConnection::CancelAndClose(struct nfsfh *fh, NfsCallback &callback)
+NfsConnection::CancelAndClose(struct nfsfh *fh, NfsCallback &callback) noexcept
 {
 	CancellableCallback &cancel = callbacks.Get(callback);
 	cancel.CancelAndScheduleClose(fh);
 }
 
 void
-NfsConnection::DestroyContext()
+NfsConnection::DestroyContext() noexcept
 {
 	assert(GetEventLoop().IsInside());
 	assert(context != nullptr);
@@ -364,13 +394,13 @@ NfsConnection::DestroyContext()
 #endif
 
 	if (!mount_finished) {
-		assert(TimeoutMonitor::IsActive());
-		TimeoutMonitor::Cancel();
+		assert(mount_timeout_event.IsActive());
+		mount_timeout_event.Cancel();
 	}
 
-	/* cancel pending DeferredMonitor that was scheduled to notify
+	/* cancel pending DeferEvent that was scheduled to notify
 	   new leases */
-	DeferredMonitor::Cancel();
+	defer_new_lease.Cancel();
 
 	if (SocketMonitor::IsDefined())
 		SocketMonitor::Steal();
@@ -384,7 +414,7 @@ NfsConnection::DestroyContext()
 }
 
 inline void
-NfsConnection::DeferClose(struct nfsfh *fh)
+NfsConnection::DeferClose(struct nfsfh *fh) noexcept
 {
 	assert(GetEventLoop().IsInside());
 	assert(in_event);
@@ -396,7 +426,7 @@ NfsConnection::DeferClose(struct nfsfh *fh)
 }
 
 void
-NfsConnection::ScheduleSocket()
+NfsConnection::ScheduleSocket() noexcept
 {
 	assert(GetEventLoop().IsInside());
 	assert(context != nullptr);
@@ -413,11 +443,11 @@ NfsConnection::ScheduleSocket()
 		SocketMonitor::Steal();
 
 	if (!SocketMonitor::IsDefined()) {
-		int _fd = nfs_get_fd(context);
-		if (_fd < 0)
+		SocketDescriptor _fd(nfs_get_fd(context));
+		if (!_fd.IsDefined())
 			return;
 
-		fd_set_cloexec(_fd, true);
+		_fd.EnableCloseOnExec();
 		SocketMonitor::Open(_fd);
 	}
 
@@ -426,7 +456,7 @@ NfsConnection::ScheduleSocket()
 }
 
 inline int
-NfsConnection::Service(unsigned flags)
+NfsConnection::Service(unsigned flags) noexcept
 {
 	assert(GetEventLoop().IsInside());
 	assert(context != nullptr);
@@ -451,7 +481,7 @@ NfsConnection::Service(unsigned flags)
 }
 
 bool
-NfsConnection::OnSocketReady(unsigned flags)
+NfsConnection::OnSocketReady(unsigned flags) noexcept
 {
 	assert(GetEventLoop().IsInside());
 	assert(deferred_close.empty());
@@ -523,15 +553,15 @@ NfsConnection::OnSocketReady(unsigned flags)
 
 inline void
 NfsConnection::MountCallback(int status, gcc_unused nfs_context *nfs,
-			     gcc_unused void *data)
+			     gcc_unused void *data) noexcept
 {
 	assert(GetEventLoop().IsInside());
 	assert(context == nfs);
 
 	mount_finished = true;
 
-	assert(TimeoutMonitor::IsActive() || in_destroy);
-	TimeoutMonitor::Cancel();
+	assert(mount_timeout_event.IsActive() || in_destroy);
+	mount_timeout_event.Cancel();
 
 	if (status < 0) {
 		auto e = FormatRuntimeError("nfs_mount_async() failed: %s",
@@ -543,7 +573,7 @@ NfsConnection::MountCallback(int status, gcc_unused nfs_context *nfs,
 
 void
 NfsConnection::MountCallback(int status, nfs_context *nfs, void *data,
-			     void *private_data)
+			     void *private_data) noexcept
 {
 	NfsConnection *c = (NfsConnection *)private_data;
 
@@ -563,7 +593,7 @@ NfsConnection::MountInternal()
 	postponed_mount_error = std::exception_ptr();
 	mount_finished = false;
 
-	TimeoutMonitor::Schedule(NFS_MOUNT_TIMEOUT);
+	mount_timeout_event.Schedule(NFS_MOUNT_TIMEOUT);
 
 #ifndef NDEBUG
 	in_service = false;
@@ -584,7 +614,7 @@ NfsConnection::MountInternal()
 }
 
 void
-NfsConnection::BroadcastMountSuccess()
+NfsConnection::BroadcastMountSuccess() noexcept
 {
 	assert(GetEventLoop().IsInside());
 
@@ -596,7 +626,7 @@ NfsConnection::BroadcastMountSuccess()
 }
 
 void
-NfsConnection::BroadcastMountError(std::exception_ptr &&e)
+NfsConnection::BroadcastMountError(std::exception_ptr &&e) noexcept
 {
 	assert(GetEventLoop().IsInside());
 
@@ -610,7 +640,7 @@ NfsConnection::BroadcastMountError(std::exception_ptr &&e)
 }
 
 void
-NfsConnection::BroadcastError(std::exception_ptr &&e)
+NfsConnection::BroadcastError(std::exception_ptr &&e) noexcept
 {
 	assert(GetEventLoop().IsInside());
 
@@ -624,7 +654,7 @@ NfsConnection::BroadcastError(std::exception_ptr &&e)
 }
 
 void
-NfsConnection::OnTimeout()
+NfsConnection::OnMountTimeout() noexcept
 {
 	assert(GetEventLoop().IsInside());
 	assert(!mount_finished);
@@ -636,7 +666,7 @@ NfsConnection::OnTimeout()
 }
 
 void
-NfsConnection::RunDeferred()
+NfsConnection::RunDeferred() noexcept
 {
 	assert(GetEventLoop().IsInside());
 
